#include <iostream>
#include <vector>
#include <unordered_set>
#include <cmath>
#include <cassert>
#define de(x) cout << #x << " = " << x << endl;
using namespace std;
typedef long long ll;
const ll MAX_L = 20;
const ll MAX_N = 1 << MAX_L;

ll A[MAX_N];
ll B[MAX_N];
ll A_prime[MAX_N];

// Sparse XOR convolution
void sparse_xor_convolution(const vector<ll>& indices, vector<ll>& result) {
    for (size_t i = 0; i < indices.size(); i++) {
        for (size_t j = 0; j < indices.size(); j++) {
            ll x = indices[i] ^ indices[j];
            result[x]++;
        }
    }
}

// Fast Walsh-Hadamard Transform (FWHT)
void fwht(ll* A, bool inverse = false) {
    ll n = 1 << MAX_L;
    for (ll len = 1; 2 * len <= n; len *= 2) {
        for (ll i = 0; i < n; i += 2 * len) {
            for (ll j = 0; j < len; j++) {
                ll u = A[i + j];
                ll v = A[i + j + len];
                A[i + j] = u + v;
                A[i + j + len] = u - v;
            }
        }
    }
    if (inverse) {
        for (ll i = 0; i < n; i++) {
            A[i] /= n;
        }
    }
}

int main() {
    ll n, l;
    cin >> n >> l;

    for (ll i = 0; i < n; i++) {
        ll x;
        cin >> x;
        assert(0 <= x && x < (1 << l));
        A[x]++;
    }

    unordered_set<ll> unique_values;
    for (ll i = 0; i < MAX_N; i++) {
        if (A[i] > 0) {
            unique_values.insert(A[i]);
        }
    }

    fill(B, B + MAX_N, 0);

    for (auto val : unique_values) {
        fill(A_prime, A_prime + MAX_N, 0);

        ll cnt = 0;
        vector<ll> indices;  // To store non-zero indices for sparse convolution
        for (ll i = 0; i < MAX_N; i++) {
            if (A[i] == val) {
                A_prime[i] = 1;
                indices.push_back(i);
                cnt++;
            }
        }

        if (indices.size() < sqrt(n)) {
            // Use sparse XOR convolution
            vector<ll> temp(MAX_N);
            sparse_xor_convolution(indices, temp);
            for (ll i = 0; i < MAX_N; i++) {
                A_prime[i] = temp[i];
            }
        } else {
            // Use FWHT for dense arrays
            fwht(A_prime);
            for (ll i = 0; i < MAX_N; i++) {
                A_prime[i] *= A_prime[i];
            }
            fwht(A_prime, true);
        }

        for (ll i = 0; i < MAX_N; i++) {
            B[i] += (cnt == A_prime[i]);
        }
    }

    ll valid_x_count = 0;
    for (ll i = 0; i < MAX_N; i++) {
        valid_x_count += (B[i] == unique_values.size());
    }
	de(valid_x_count);
	de(unique_values.size());
	cout << "unique_values: "; for(auto x : unique_values) cout << x << " "; cout << endl;
    cout << (1ll << l) / valid_x_count << endl;
    return 0;
}

